package org.king.modules.message.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.king.common.api.vo.Result;
import org.king.common.system.base.controller.KingController;
import org.king.common.system.query.QueryGenerator;
import org.king.modules.message.entity.SysMessage;
import org.king.modules.message.service.ISysMessageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

/**
 * @Description: 消息
 * @Author: king-boot
 * @Date: 2019-04-09
 * @Version: V1.0
 */
@Slf4j
@RestController
@RequestMapping("/message/sysMessage")
@Api(tags = "发送消息管理")
public class SysMessageController extends KingController<SysMessage, ISysMessageService> {
    @Autowired
    private ISysMessageService sysMessageService;

    /**
     * 分页列表查询
     *
     * @param sysMessage
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    @ApiOperation("分页列表查询")
    public Result<?> queryPageList(SysMessage sysMessage, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize, HttpServletRequest req) {
        QueryWrapper<SysMessage> queryWrapper = QueryGenerator.initQueryWrapper(sysMessage, req.getParameterMap());
        Page<SysMessage> page = new Page<>(pageNo, pageSize);
        IPage<SysMessage> pageList = sysMessageService.page(page, queryWrapper);
        return Result.ok(pageList);
    }

    /**
     * 添加
     *
     * @param sysMessage
     * @return
     */
    @PostMapping(value = "/add")
    @ApiOperation("添加")
    public Result<?> add(@RequestBody SysMessage sysMessage) {
        sysMessageService.save(sysMessage);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     *
     * @param sysMessage
     * @return
     */
    @PutMapping(value = "/edit")
    @ApiOperation("编辑")
    public Result<?> edit(@RequestBody SysMessage sysMessage) {
        sysMessageService.updateById(sysMessage);
        return Result.ok("修改成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    @ApiOperation("通过id删除")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        sysMessageService.removeById(id);
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    @ApiOperation("批量删除")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.sysMessageService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    @ApiOperation("通过id查询")
    public Result<?> queryById(@RequestParam(name = "id", required = true) String id) {
        SysMessage sysMessage = sysMessageService.getById(id);
        return Result.ok(sysMessage);
    }

    /**
     * 导出excel
     *
     * @param request
     */
    @GetMapping(value = "/exportXls")
    @ApiOperation("导出")
    public ModelAndView exportXls(HttpServletRequest request, SysMessage sysMessage) {
        return super.exportXls(request, sysMessage, SysMessage.class, "推送消息模板");
    }

    /**
     * excel导入
     *
     * @param request
     * @param response
     * @return
     */
    @PostMapping(value = "/importExcel")
    @ApiOperation("导入")
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, SysMessage.class);
    }

}
