package org.king.modules.message.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.king.common.api.vo.Result;
import org.king.common.system.base.controller.KingController;
import org.king.common.system.query.QueryGenerator;
import org.king.modules.message.entity.MsgParams;
import org.king.modules.message.entity.SysMessageTemplate;
import org.king.modules.message.service.ISysMessageTemplateService;
import org.king.modules.message.util.PushMsgUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Map;

/**
 * @Description: 消息模板
 * @Author: king-boot
 * @Date: 2019-04-09
 * @Version: V1.0
 */
@Slf4j
@RestController
@RequestMapping("/message/sysMessageTemplate")
@Api(tags = "消息模板管理")
public class SysMessageTemplateController extends KingController<SysMessageTemplate, ISysMessageTemplateService> {
    @Autowired
    private ISysMessageTemplateService sysMessageTemplateService;
    @Autowired
    private PushMsgUtil pushMsgUtil;

    /**
     * 分页列表查询
     *
     * @param sysMessageTemplate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    @ApiOperation("分页列表查询")
    public Result<?> queryPageList(SysMessageTemplate sysMessageTemplate, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize, HttpServletRequest req) {
        QueryWrapper<SysMessageTemplate> queryWrapper = QueryGenerator.initQueryWrapper(sysMessageTemplate, req.getParameterMap());
        Page<SysMessageTemplate> page = new Page<>(pageNo, pageSize);
        IPage<SysMessageTemplate> pageList = sysMessageTemplateService.page(page, queryWrapper);
        return Result.ok(pageList);
    }

    /**
     * 添加
     *
     * @param sysMessageTemplate
     * @return
     */
    @PostMapping(value = "/add")
    @ApiOperation("添加")
    public Result<?> add(@RequestBody SysMessageTemplate sysMessageTemplate) {
        sysMessageTemplateService.save(sysMessageTemplate);
        return Result.ok("添加成功！");
    }

    /**
     * 编辑
     *
     * @param sysMessageTemplate
     * @return
     */
    @PutMapping(value = "/edit")
    @ApiOperation("编辑")
    public Result<?> edit(@RequestBody SysMessageTemplate sysMessageTemplate) {
        sysMessageTemplateService.updateById(sysMessageTemplate);
        return Result.ok("更新成功！");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    @ApiOperation("通过id删除")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        sysMessageTemplateService.removeById(id);
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    @ApiOperation("批量删除")
    public Result<?> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.sysMessageTemplateService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.ok("批量删除成功！");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    @ApiOperation("通过id查询")
    public Result<?> queryById(@RequestParam(name = "id") String id) {
        SysMessageTemplate sysMessageTemplate = sysMessageTemplateService.getById(id);
        return Result.ok(sysMessageTemplate);
    }

    /**
     * 导出excel
     *
     * @param request
     */
    @GetMapping(value = "/exportXls")
    @ApiOperation("导出excel")
    public ModelAndView exportXls(HttpServletRequest request, SysMessageTemplate sysMessageTemplate) {
        return super.exportXls(request, sysMessageTemplate, SysMessageTemplate.class, "推送消息模板");
    }

    /**
     * excel导入
     *
     * @param request
     * @param response
     * @return
     */
    @PostMapping(value = "/importExcel")
    @ApiOperation("excel导入")
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, SysMessageTemplate.class);
    }

    /**
     * 发送消息
     */
    @PostMapping(value = "/sendMsg")
    @ApiOperation("发送消息")
    public Result<SysMessageTemplate> sendMessage(@RequestBody MsgParams msgParams) {
        Result<SysMessageTemplate> result = new Result<>();
        Map<String, String> map = null;
        try {
            map = (Map<String, String>) JSON.parse(msgParams.getTestData());
        } catch (Exception e) {
            result.error500("解析Json出错！");
            return result;
        }
        boolean isSendsuccess = pushMsgUtil.sendMessage(msgParams.getMsgType(), msgParams.getTemplateCode(), map, msgParams.getReceiver());
        if (isSendsuccess) {
            result.success("发送消息任务添加成功！");
        } else {
            result.error500("发送消息任务添加失败！");
        }
        return result;
    }
}
