package org.king.modules.system.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.king.common.api.vo.Result;
import org.king.common.system.query.QueryGenerator;
import org.king.modules.system.entity.SysDataLog;
import org.king.modules.system.service.ISysDataLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 数据日志比较
 * @Author: king-boot
 * @Date: 2019-05-29
 * @Version: V1.0
 */
@RestController
@RequestMapping("/sys/dataLog")
@Slf4j
@Api(tags = "数据日志比较")
public class SysDataLogController {
    @Autowired
    private ISysDataLogService service;

    @GetMapping(value = "/list")
    @ApiOperation("查询数据列表")
    public Result<IPage<SysDataLog>> queryPageList(SysDataLog dataLog, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize, HttpServletRequest req) {
        Result<IPage<SysDataLog>> result = new Result<>();
        QueryWrapper<SysDataLog> queryWrapper = QueryGenerator.initQueryWrapper(dataLog, req.getParameterMap());
        Page<SysDataLog> page = new Page<>(pageNo, pageSize);
        IPage<SysDataLog> pageList = service.page(page, queryWrapper);
        log.info("查询当前页：" + pageList.getCurrent());
        log.info("查询当前页数量：" + pageList.getSize());
        log.info("查询结果数量：" + pageList.getRecords().size());
        log.info("数据总数：" + pageList.getTotal());
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 查询对比数据
     *
     * @param req
     * @return
     */
    @GetMapping(value = "/queryCompareList")
    @ApiOperation("查询对比数据")
    public Result<List<SysDataLog>> queryCompareList(HttpServletRequest req) {
        Result<List<SysDataLog>> result = new Result<>();
        String dataId1 = req.getParameter("dataId1");
        String dataId2 = req.getParameter("dataId2");
        List<String> idList = new ArrayList<>();
        idList.add(dataId1);
        idList.add(dataId2);
        try {
            List<SysDataLog> list = (List<SysDataLog>) service.listByIds(idList);
            result.setResult(list);
            result.setSuccess(true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return result;
    }

    /**
     * 查询版本信息
     *
     * @param req
     * @return
     */
    @GetMapping(value = "/queryDataVerList")
    @ApiOperation("查询版本信息")
    public Result<List<SysDataLog>> queryDataVerList(HttpServletRequest req) {
        Result<List<SysDataLog>> result = new Result<>();
        String dataTable = req.getParameter("dataTable");
        String dataId = req.getParameter("dataId");
        QueryWrapper<SysDataLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("data_table", dataTable);
        queryWrapper.eq("data_id", dataId);
        List<SysDataLog> list = service.list(queryWrapper);
        if (list == null || list.size() <= 0) {
            result.error500("未找到版本信息");
        } else {
            result.setResult(list);
            result.setSuccess(true);
        }
        return result;
    }

}
